@@ -1,10 +1,11 @@
 from demyst.analytics.report import report
+from demyst.report.corr import corr_compute
 
 import matplotlib.pyplot as plt
 from io import BytesIO
-import pdfkit
 import pandas as pd
 import numpy as np
+import seaborn as sns
 import os
 import shutil
 from matplotlib import font_manager
@@ -14,6 +15,10 @@ import platform
 from yattag import Doc
 import re
 
+import warnings
+warnings.filterwarnings("ignore", 'invalid value encountered')
+warnings.filterwarnings("ignore", 'divide by zero encountered in double_scalars')
+
 # This is a Windows workaround for a bug in wkhtmltopdf:
 # https://github.com/wkhtmltopdf/wkhtmltopdf/issues/3081
 # On Unix-like systems, use `ulimit -n 2048`.
@@ -50,9 +55,13 @@ def quality_report(analytics, df=None, rep=None):
     distribution_diagrams(analytics, rep, all_stats)
     generate_html(analytics, df, rep, all_stats)
     copy_files(here_dir)
-    pdfkit.from_file("report/index.html",
-                     "report/report.pdf",
-                     { 'margin-top': '0px', 'margin-left': '0px', 'margin-bottom': '0px', 'margin-right': '0px' })
+    pdf_from_html("report/index.html", "report/report.pdf")
+    print("Done.")
+
+def pdf_from_html(html_file, pdf_file):
+    ret = os.system("wkhtmltopdf -L 0mm -T 0mm -R 0mm -B 0mm {} {}".format(html_file, pdf_file))
+    if ret != 0:
+        raise RuntimeError("wkhtmltopdf error: {}".format(ret))
 
 def save_figure(filename):
     plt.savefig("report/" + filename + ".svg", format='svg', bbox_inches='tight', transparent=True)
@@ -78,6 +87,7 @@ def data_stats_colors():
     return ["#b39ddb", "#1de9B6", "#2e3951", "#00acc1", "#7283a7"]
     
 def data_stats_diagram(analytics, rep):
+    print("Creating data stats diagram")
     types, values = data_stats(analytics, rep)
     patches, texts = plt.pie(values, colors=data_stats_colors())
     return save_figure("data_stats")
@@ -96,6 +106,11 @@ def generate_html(analytics, df, rep, all_stats):
             page_break(doc)
             header(doc)
             data_stats_section(analytics, rep, doc)
+            # Only plot correlations if we have a DF
+            if isinstance(df, pd.DataFrame) and (len(df) >= 2):
+                page_break(doc)
+                header(doc)
+                correlations_section(analytics, df, rep, doc)
             page_break(doc)
             header(doc)
             attr_stats_section(analytics, rep, doc)
@@ -120,6 +135,44 @@ def cover(doc):
             doc.line("div", "Quality")
             doc.line("div", "Report", klass="dq")
 
+def correlations_section(a, df, rep, doc):
+    corr_matrixes = corr_compute(df, rep)
+    res = pd.DataFrame({})
+    for name, matrix in corr_matrixes.items():
+        print("Threshold computation: " + name)
+        df_highcorr = threshold_computation(df, matrix)
+        # Merge df into result, ignoring any columns already in result
+        res = pd.concat([res, df_highcorr.drop(list(res), axis="columns", errors="ignore")], sort=False)
+    plot_correlations(res)
+    doc.line("h2", "Correlations")
+    doc.stag("img", src="corrplot.svg", width="100%")
+
+def threshold_computation(df, corr_matrix):
+    upper = corr_matrix.where(np.triu(np.ones(corr_matrix.shape), k=1).astype(np.bool))
+    threshold = 0.95
+    to_keep = [column for column in upper.columns if any(upper[column] > 0.90)]
+    while True:
+        if len(to_keep) < 31:
+            break
+        if threshold == 0.998:
+            to_keep = [column for column in upper.columns[0:29]]
+            break
+        threshold = threshold + 0.001
+        to_keep = [column for column in upper.columns if any(upper[column] > threshold)]
+    df_highcorr = df[to_keep]
+    return df_highcorr
+
+def plot_correlations(df):
+    corrmat = df.corr().abs()
+    # Drop all-NaN rows and columns
+    corrmat = corrmat.dropna(axis="columns", how="all")
+    corrmat = corrmat.dropna(axis="rows", how="all")
+    mask = np.zeros_like(corrmat)
+    mask[np.triu_indices_from(mask)] = True
+    plt.subplots(figsize=(12, 9))
+    sns.heatmap(corrmat, square=True, cmap="YlGnBu", linewidths=.5, mask=mask, cbar=False)
+    save_figure("corrplot")
+
 def legend_item(doc, color):
     doc.line("span", NBSP, klass="legend_item", style="background-color: " + color)
 
@@ -173,7 +226,10 @@ def product_stats_section(a, df, rep, all_stats, doc):
             doc.line("td", str(rep["matches_all"].values[0]))
 
 def product_stats_from_all_stats(all_stats, pid):
-    return all_stats.loc[all_stats["product_name"] == pid].sort_values("stats_updated_on", ascending=False)
+    if all_stats.empty:
+        return pd.DataFrame({})
+    else:
+        return all_stats.loc[all_stats["product_name"] == pid].sort_values("stats_updated_on", ascending=False)
 
 def product_stat_bar(doc, match_rate, error_rate):
     with doc.tag("table", klass="product_stats_diagram", cellpadding="0", cellspacing="0"):
@@ -190,6 +246,7 @@ def product_stat_bar(doc, match_rate, error_rate):
                 doc.line("td", s, width=s, style="background-color: #b39ddb")
 
 def distribution_diagrams(a, rep, all_stats):
+    print("Creating distribution diagrams")
     products = provider_names_from_report(rep)
     for p in products:
         # per-job hist
